#include <iostream>

#include "Rectangle/ContourAnalyzer.h"
#include "webcam/Webcam.h"
#include "UI.h"
#include "Rectangle/Confidence.h"
#include "Rectangle/BoundTracker.h"

enum State {
	detection, track, cross_check
};

State state = detection;
Webcam webcam = 0;
int *lastKey = new int(0);
UI *ui = new UI(lastKey);
ContourAnalyzer ca { ui, lastKey };
Confidence conf { lastKey, ui, 5};
BoundTracker tracker {lastKey, ui};
cv::Point statusTextPos{5, 25};
cv::Scalar textFontColor{35, 125, 35};
cv::Scalar rectangleBoundColor{255, 0, 0};

unsigned frameCount = 0;
void run() {
	switch (state) {
	case detection:
		cv::putText(*(ui->drawnFrame()), "Looking for rectangle...", statusTextPos, cv::FONT_HERSHEY_PLAIN, 0.7, 35);
		ca.analyze();
		ca.convertContoursToBounds();
		if (conf.check(ca.getBounds())) {
			cv::Rect largest;
			std::vector<cv::Rect> rects = conf.rectangleBounds();
			for (unsigned i = 0; i < rects.size(); i++) {
				if (rects[i].area() > largest.area()) {
					largest = rects[i];
				}
			}

			tracker.init(largest);

			state = track;
		}
		break;

	case track:
		cv::putText(*(ui->drawnFrame()), "Tracking...", statusTextPos, cv::FONT_HERSHEY_PLAIN, 0.7, 35);
		if (!tracker.track()) {
			state = cross_check;
		}
		cv::rectangle(*(ui->drawnFrame()), tracker.getROI(), cv::Scalar{255, 0, 0}, 2);
		break;

	case cross_check:
		cv::putText(*(ui->drawnFrame()), "Cross checking...", statusTextPos, cv::FONT_HERSHEY_PLAIN, 0.7, 35);
		ca.analyze();
		ca.convertContoursToBounds();
		cv::Rect2d largest;
		std::vector<cv::Rect> rects = ca.getBounds();
		for (unsigned i = 0; i < rects.size(); i++) {
			if (rects[i].area() > largest.area()) {
				largest = rects[i];
			}
		}

		tracker.track();

		unsigned largestArea = tracker.getROI().area() > largest.area() ? tracker.getROI().area() : largest.area();
		if (largest.area() != 0 && (tracker.getROI() & largest).area() >= largestArea*(0.75)) {
			state = track;
		} else {
			frameCount++;
			if (frameCount > 15) {
				state = detection;
				frameCount = 0;
			}
		}
		cv::rectangle(*(ui->drawnFrame()), tracker.getROI(), cv::Scalar{255, 0, 0}, 2);
		cv::rectangle(*(ui->drawnFrame()), largest, cv::Scalar{255, 30, 30}, 1);

		break;
	}
	ui->render();
}

int main(int argc, char** argv) {
	while (true) {
		webcam.update();
		cv::Mat frame = webcam.getFrame();
		ui->setOriginalFrame(frame);

		*lastKey = cv::waitKey(33);
		if (*lastKey > 0) {
			std::cout << "key pressed: " << *lastKey << std::endl;
			if (*lastKey == 27) {
				break;
			}
		}
		run();
		*lastKey = -1;
	}

	delete ui;
	delete lastKey;
	cv::destroyAllWindows();
	return 0;
}
